//description file converter (UltraStar TEXT to SStar XML)

#if !defined(SSTARMOD_DESC_H)
#define SSTARMOD_DESC_H

namespace desc {
   using namespace conv;

   typedef double        BPM;
   typedef unsigned long BEATS;
   typedef unsigned long MIDINOTE;

   //sstar uses differently encoded xml files
   //strangely enough, the files i examined, have had the encoding
   //  attribute set to different formats, but anyhow they were
   //  all encoded to ISO-8859-1
   enum CONVFORMAT {
      CONVFORMAT__UTF_8,
      CONVFORMAT__UTF_16,
      CONVFORMAT__ISO_8859_1,
   };

   ///PARSER BASE CLASS
   class ParserBaseC : public ConvBaseC {
      private:
         typedef ConvBaseC INH;

      public:
         ///constructor, destructor
                 ParserBaseC() { }
                 //ParserBaseC(const wxFileName& filename) : ConvBaseC(filename) { } todo
                 //ParserBaseC(const wxString& filename)   : ConvBaseC(filename) { }
         virtual ~ParserBaseC() { }

         ///overloaded
         virtual int ReadHeader();
         virtual int Parse();
         virtual int OpenAndVerify();
   };

   ///PARSER FOR ULTRASTAR TEXT DESCRIPTION FILES
   class TextParserC : public ParserBaseC {
      private:
         typedef ParserBaseC INH;

         wxTextFile textfileSrc;

      public:
         struct TextEntryT {
            enum TEXTENTRYTYPE {
               TEXTENTRYTYPE_SYLLABLE, //contains a syllable
               TEXTENTRYTYPE_LINEINFO, //contains timing information for beginning with next line
            };

            TEXTENTRYTYPE nType;

            //for syllable information
            BEATS     nStart;
            BEATS     nDuration;
            MIDINOTE  nMidiNote;
            wxString* strSyllable; //has to be ptr, size is not known yet

            //for text line timing information, todo: needed?
            BEATS nLineClear;
            BEATS nLineStart;
         };

         ///overloaded
         virtual int  OpenAndVerify();
         virtual int  ReadHeader(); //header and meta information
         virtual int  Parse();
         virtual void Close();

         ///constructor, destructor
         TextParserC() : ParserBaseC() { }
         ~TextParserC() { } //todo: destructor
         TextParserC(const wxFileName& filename) : ParserBaseC() { ParserBaseC::fnmSrc = filename;
                                                                   OpenAndVerify();
                                                                 }
         TextParserC(const wxString& filename) :   ParserBaseC() { ParserBaseC::fnmSrc = filename;
                                                                   OpenAndVerify();
                                                                 }

         ///members
         TextEntryT* Entry;     //pointer to array holding entry informations
         size_t      nEntryCnt; //array size

         //header information
         wxString      strTitle;
         wxString      strArtist;
         BPM           nBpm;
         unsigned long nGapAudio;
         unsigned long nGapVid;
         wxString      strTargetAudio;
         wxString      strTargetVid;
         bool          bRelative;
   };

   ///PARSER FOR SSTAR XML LYRIC DESCRIPTION FILES
   class XmlParserC : public ParserBaseC {
      private:
         typedef ParserBaseC INH;

         ///members
         XMLNode xMain;

      public:
         ///overloaded
         int ReadHeader();
         int OpenAndVerify();

         ///constructor, destructor
         XmlParserC() : ParserBaseC() { }
         ~XmlParserC() { } //todo: destructor
         XmlParserC(const wxFileName& filename) : ParserBaseC() { ParserBaseC::fnmSrc = filename;
                                                                   OpenAndVerify();
                                                                }
         XmlParserC(const wxString& filename) :   ParserBaseC() { ParserBaseC::fnmSrc =
                                                                     wxFileName(filename);
                                                                  OpenAndVerify();
                                                                }


         ///members
         wxString strArtist;
         wxString strGenre;
         long     nYear;
         BPM      nBpm;
   };

   ///PARSED ULTRASTAR TEXT => SSTAR XML DESCRIPTION CONVERTER
   class TextConvC : public ConvBaseC {
      private:
         typedef ConvBaseC INH;

      public:
         ///overloaded
         virtual int  OpenAndVerify();
         virtual int  ReadHeader();
         virtual int  Conv();
         virtual void Close();
         virtual int  WriteToFile();

         ///constructor, destructor
         TextConvC() : ConvBaseC() { }
         TextConvC(const wxFileName& filename) : ConvBaseC() { ConvBaseC::fnmSrc = filename;
                                                               OpenAndVerify();
                                                             }
         TextConvC(const wxString& filename) :   ConvBaseC() { ConvBaseC::fnmSrc = filename;
                                                               OpenAndVerify();
                                                             }
         ~TextConvC();

         ///members
         TextParserC TextParser;
         XMLSTR      bufDest;
         int         nBufDestLen;

         //required information
         long     nYear;
         wxString strGenre;

         //optional information, if NULL it will be taken from the source file
         wxString strOptArtist;
   };
}

#endif //SSTARMOD_DESC_H
