/*
gui todo:
   - auto detect filetype
   - settings
*/

#include "main.h"

namespace gui {

/**************
  MAIN WINDOW
**************/

/*
   abbrev.: M:  Main
            V:  Vertical
            ly: layout (sizer ctrls)
            B:  Box (box for sections)
*/

gui::frmMain* frame;

///EVENT TABLE
BEGIN_EVENT_TABLE(frmMain, wxFrame)
   EVT_BUTTON              (frmMain::ID_BtnPAK,     frmMain::OnBtnPAK)
   EVT_RADIOBUTTON         (frmMain::ID_RadPAK,     frmMain::OnRadPAK)
   EVT_BUTTON              (frmMain::ID_BtnDest,    frmMain::OnBtnDest)
   EVT_BUTTON              (frmMain::ID_BtnSrcAdd,  frmMain::OnBtnSrcAdd)
   EVT_BUTTON              (frmMain::ID_BtnSrcDel,  frmMain::OnBtnSrcDel)
   EVT_BUTTON              (frmMain::ID_BtnSrcUp,   frmMain::OnBtnSrcUp)
   EVT_BUTTON              (frmMain::ID_BtnSrcDown, frmMain::OnBtnSrcDown)
   EVT_LIST_ITEM_ACTIVATED (frmMain::ID_LstSrc,     frmMain::OnLstSrc)
   EVT_LIST_ITEM_SELECTED  (frmMain::ID_LstSrc,     frmMain::OnLstSrcSel)
END_EVENT_TABLE()

///CREATE WINDOW
frmMain::frmMain(const wxString& title)
   : wxFrame(NULL, wxID_ANY, title)
{
   ///main
   wxBoxSizer* lyM = new wxBoxSizer(wxVERTICAL);

   ///status bar
   this->CreateStatusBar(1, wxST_SIZEGRIP, wxID_ANY);

   ///source of PAK files
   wxBoxSizer* lyPAK = new wxBoxSizer(wxHORIZONTAL);
   lyM->Add(new wxStaticText(this, wxID_ANY, _T("Select Folder of PAK Files")), 0, wxALL, 5);
   lyM->Add(lyPAK, 0, wxEXPAND | wxALIGN_TOP);
   this->txtPAK = new wxTextCtrl(this, wxID_ANY);
   lyPAK->Add(this->txtPAK, 1, wxALL ^ wxTOP, 5);
   this->btnPAK = new wxButton(this, frmMain::ID_BtnPAK, _T("..."));
   lyPAK->Add(this->btnPAK, 0, wxALL ^ wxTOP, 5);
   this->radPAK = new wxRadioButton(this, frmMain::ID_RadPAK, _T("Folder contains extracted files"));
   radPAK->SetValue(true);
   lyM->Add(new wxRadioButton(this, wxID_ANY, _T("Folder contains packed PAK files")), 0,
                              wxALL ^ wxBOTTOM ^ wxTOP, 5);
   lyM->Add(radPAK, 0, wxALL, 5);

   ///destination
   wxBoxSizer* lyDest = new wxBoxSizer(wxHORIZONTAL);
   lyM->Add(new wxStaticText(this, wxID_ANY, _T("Select Destination Folder")), 0,
                             wxALL ^ wxBOTTOM, 5);
   lyM->Add(lyDest, 0, wxEXPAND | wxALIGN_TOP | wxALL, 2);
   this->txtDest = new wxTextCtrl(this, wxID_ANY);
   lyDest->Add(this->txtDest, 1, wxALL, 5);
   this->btnDest = new wxButton(this, frmMain::ID_BtnDest, _T("..."));
   lyDest->Add(this->btnDest, 0, wxALL, 5);

   ///source of content to add
   lyM->Add(new wxStaticText(this, wxID_ANY, _T("Select Files To Add")), 0, wxALL ^ wxBOTTOM, 5);
   this->lstSrc = new wxListCtrl(this, ID_LstSrc, wxDefaultPosition, wxDefaultSize, wxLC_REPORT);
   this->lstSrc->InsertColumn(0, _T("Index"));
   this->lstSrc->InsertColumn(1, _T("File"));
   lyM->Add(this->lstSrc, 1, wxEXPAND | wxALL ^ wxBOTTOM, 5);
   wxBoxSizer* lySrc = new wxBoxSizer(wxHORIZONTAL);
   lyM->Add(lySrc, 0, wxEXPAND | wxALIGN_TOP | wxALL, 5);
   lySrc->Add(new wxButton(this, frmMain::ID_BtnSrcAdd, _T("Add")), 1, wxRIGHT, 2);
   lySrc->Add(new wxButton(this, frmMain::ID_BtnSrcDel, _T("Delete")), 1, wxRIGHT, 2);
   lySrc->Add(new wxButton(this, frmMain::ID_BtnSrcUp, _T("Move Up")), 1, wxRIGHT, 2);
   lySrc->Add(new wxButton(this, frmMain::ID_BtnSrcDown, _T("Move Down")), 1);

   SetSizer(lyM);
}

///EVENT HANDLERS

//btnPAK
void frmMain::OnBtnPAK(wxCommandEvent& event) {
   wxDirDialog dlgDir(this, _T("Select Folder Containing PAK Files"));
   if(dlgDir.ShowModal() == wxID_OK)
      this->txtPAK->SetValue(dlgDir.GetPath());
}

//radPAK
void frmMain::OnRadPAK(wxCommandEvent& event) {
   this->bRadPAKEx = this->radPAK->GetValue();
}

//btnDest
void frmMain::OnBtnDest(wxCommandEvent& event) {
   wxDirDialog dlgDir(this, _T("Select Destination Folder"));
   if(dlgDir.ShowModal() == wxID_OK)
      this->txtDest->SetValue(dlgDir.GetPath());
}

//activate item in source list
void frmMain::OnLstSrc(wxListEvent& event) {
   dlgSettingsC dlgSettings(this, wxID_ANY, _T("Settings"));
   dlgSettings.SetSettings((::SettingsC*) event.GetData());
   if(dlgSettings.ShowModal() == wxID_OK) {
      ::SettingsC* Settings = dlgSettings.GetSettings();
      int nPos = event.GetIndex();
      this->lstSrc->SetItem(nPos, 1, wxString(Settings->strArtist)<< _T(" - ")
                            << Settings->strTitle);
      //wxString str;
      //str.Printf(_T("%02d"), nPos);
      //this->lstSrc->SetItem(nPos, 0, str);
   }
}

//select item in source list
void frmMain::OnLstSrcSel(wxListEvent& event) {
   this->nLstSrcSel = event.GetIndex();
   wxLogMessage(_T("gui: selected item in LstSrc, Settings->nTrackIndex==%d, item selected==%d"),
                ((::SettingsC*) event.GetData())->nTrackIndex, this->nLstSrcSel);
}

//add source
void frmMain::OnBtnSrcAdd(wxCommandEvent& event) {
   int nPos = this->lstSrc->GetItemCount();

   dlgSettingsC dlgSettings(this, wxID_ANY, wxString(_T("Settings For Entry "))<< nPos);
   if(dlgSettings.ShowModal() == wxID_OK) {
      wxLogMessage(_T("gui::frmMain::OnBtnSrcAdd: dlgSettings.ShowModal() == wxID_OK"));

      ::SettingsC* Settings = dlgSettings.GetSettings();
      Settings->nTrackIndex = nPos;

      wxString str;
      str.Printf(_T("%02d"), nPos);
      this->lstSrc->InsertItem(nPos, str);
      wxLogMessage(_T("gui::frmMain::OnBtnSrcAdd: this->lstSrc->InsertItem(nPos=%d, ..."), nPos);

      this->lstSrc->SetItem(nPos, 1, wxString(Settings->strArtist)<< _T(" - ")
                            << Settings->strTitle);
      this->lstSrc->SetItemPtrData(nPos, (wxUIntPtr) dlgSettings.GetSettings());
      wxLogMessage(_T("gui::frmMain::OnBtnSrcAdd: modified item %d"), nPos);
   }
}

//delete source
void frmMain::OnBtnSrcDel(wxCommandEvent& event) {
   ::SettingsC* Settings = (::SettingsC*) this->lstSrc->GetItemData(this->nLstSrcSel);

   wxLogMessage(_T("%s"), _T("gui::frmMain::OnBtnSrcDel:"));
   wxLogMessage(_T("\tfor(int i = %d; i < %d; i++)  //..."), Settings->nTrackIndex + 1,
                this->lstSrc->GetItemCount());
   for(int i = Settings->nTrackIndex + 1; i < this->lstSrc->GetItemCount(); i++)
      (((::SettingsC*) this->lstSrc->GetItemData(i))->nTrackIndex)--;

   delete Settings;
   this->lstSrc->DeleteItem(this->nLstSrcSel);

   if(this->nLstSrcSel < this->lstSrc->GetItemCount() - 1)
      this->nLstSrcSel++;
   else
      this->nLstSrcSel--;

   this->lstSrcRefresh();
}

//move row of source list up
void frmMain::OnBtnSrcUp(wxCommandEvent& event) {
   if(this->nLstSrcSel > 0) {
      (((::SettingsC*) this->lstSrc->GetItemData(this->nLstSrcSel))->nTrackIndex)--;
      this->nLstSrcSel--;
      wxLogMessage(_T("gui::frmMain::OnBtnSrcUp: set selection to %d"), this->nLstSrcSel);
      (((::SettingsC*) this->lstSrc->GetItemData(this->nLstSrcSel))->nTrackIndex)++;
      wxLogMessage(_T("%s"), _T("gui::frmMain::OnBtnSrcUp: exchanged nTrackIndex"));
      this->lstSrc->SortItems(lstSrcCmp, 0);
      this->lstSrcRefresh();
   }
}

//move row of source list down
void frmMain::OnBtnSrcDown(wxCommandEvent& event) {
   if(this->nLstSrcSel < (this->lstSrc->GetItemCount() - 1)) {
      (((::SettingsC*) this->lstSrc->GetItemData(this->nLstSrcSel))->nTrackIndex)++;
      this->nLstSrcSel++;
      wxLogMessage(_T("gui::frmMain::OnBtnSrcDown: set selection to %d"), this->nLstSrcSel);
      (((::SettingsC*) this->lstSrc->GetItemData(this->nLstSrcSel))->nTrackIndex)--;
      wxLogMessage(_T("%s"), _T("gui::frmMain::OnBtnSrcDown: exchanged nTrackIndex"));
      this->lstSrc->SortItems(lstSrcCmp, 0);
      this->lstSrcRefresh();
   }
}

///MEMBERS

//resort items
int wxCALLBACK lstSrcCmp(long item1, long item2, long sortData) {
   int nTrackIndex1 = ((::SettingsC*) item1)->nTrackIndex;
   int nTrackIndex2 = ((::SettingsC*) item2)->nTrackIndex;

   if(nTrackIndex1 < nTrackIndex2)
      return -1;
   if(nTrackIndex1 > nTrackIndex2)
      return 1;

   return 0;
}

//refresh source list
void frmMain::lstSrcRefresh() {
   //refresh track index column
   wxLogMessage(_T("%s"), _T("gui::frmMain::lstSrcRefresh: refreshing track index column"));
   for(int i = 0; i < this->lstSrc->GetItemCount(); i++)
      this->lstSrc->SetItemText(i, wxString().Format(_T("%02d"), i));
}


/******************
  SETTINGS DIALOG
******************/

///EVENT TABLE
BEGIN_EVENT_TABLE(dlgSettingsC, wxDialog)
   EVT_CHECKBOX(dlgSettingsC::ID_ChkUseAudioFromVid, dlgSettingsC::OnChkUseAudioFromVid)
   EVT_BUTTON  (dlgSettingsC::ID_BtnVid,             dlgSettingsC::OnBtnVid)
   EVT_BUTTON  (dlgSettingsC::ID_BtnAudio,           dlgSettingsC::OnBtnAudio)
   EVT_BUTTON  (dlgSettingsC::ID_BtnDesc,            dlgSettingsC::OnBtnDesc)
   EVT_COMBOBOX(dlgSettingsC::ID_CmbVid,             dlgSettingsC::OnCmbVid)
   EVT_COMBOBOX(dlgSettingsC::ID_CmbAudio,           dlgSettingsC::OnCmbAudio)
   EVT_COMBOBOX(dlgSettingsC::ID_CmbDesc,            dlgSettingsC::OnCmbDesc)
   EVT_BUTTON  (dlgSettingsC::ID_BtnVidHeader,       dlgSettingsC::OnBtnVidHeader)
   EVT_BUTTON  (dlgSettingsC::ID_BtnAudioHeader,     dlgSettingsC::OnBtnAudioHeader)
   EVT_BUTTON  (dlgSettingsC::ID_BtnOK,              dlgSettingsC::OnBtnOK)
   EVT_BUTTON  (dlgSettingsC::ID_BtnApply,           dlgSettingsC::OnBtnApply)
   EVT_BUTTON  (dlgSettingsC::ID_BtnCancel,          dlgSettingsC::OnBtnCancel)
END_EVENT_TABLE()

///CREATE DIALOG
dlgSettingsC::dlgSettingsC(wxWindow* parent, wxWindowID id, const wxString& title, const wxPoint& pos,
                           const wxSize& size, long style, const wxString& name)
   : wxDialog(parent, id, title, pos, size, style, name)
{
   this->Settings = new ::SettingsC();

   wxBoxSizer* lyM = new wxBoxSizer(wxVERTICAL);
   wxFlexGridSizer*  lyT;                          //temporary sizer
   wxStaticBoxSizer* lyB;                          //boxes for sections

   ///files
   lyB = new wxStaticBoxSizer(wxVERTICAL, this, _T("Files:"));
   lyM->Add(lyB, 0, wxALL | wxEXPAND, 5);
   this->chkUseAudioFromVid = new wxCheckBox(this, ID_ChkUseAudioFromVid, _T("Use audio from video"));
   lyB->Add(this->chkUseAudioFromVid, 0, wxALL, 5);
   lyT = new wxFlexGridSizer(3, 3, 5, 5);
   lyB->Add(lyT, 0, wxEXPAND | wxALL, 5);
   lyT->AddGrowableCol(1, 1);
   this->txtVid   = new wxTextCtrl(this, wxID_ANY);
   this->txtAudio = new wxTextCtrl(this, wxID_ANY);
   this->txtDesc  = new wxTextCtrl(this, wxID_ANY);
   this->btnAudio = new wxButton(this, ID_BtnAudio, _T("..."));
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Video:")), 0);
   lyT->Add(txtVid, 1, wxGROW);
   lyT->Add(new wxButton(this, ID_BtnVid, _T("...")), 0);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Audio:")), 0);
   lyT->Add(txtAudio, 1, wxGROW);
   lyT->Add(btnAudio, 0);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Desc:")), 0);
   lyT->Add(txtDesc, 1, wxGROW);
   lyT->Add(new wxButton(this, ID_BtnDesc, _T("...")), 0);

   ///file types
   lyB = new wxStaticBoxSizer(wxVERTICAL, this, _T("File Types:"));
   lyM->Add(lyB, 0, wxEXPAND | wxALL, 5);
   lyT = new wxFlexGridSizer(2, 3, 5, 5);
   for(int i = 0; i < 3; i++)
      lyT->AddGrowableCol(i, 1);
   lyB->Add(lyT, 0, wxEXPAND | wxALL | wxGROW, 5);
   this->strVidType.Add(_T("MPEG Video"));
   this->strVidType.Add(_T("AVI Video"));
   this->strVidType.Add(_T("IPU SStar Video"));
   this->cmbVid = new wxComboBox(this, ID_CmbVid, _T(""), wxDefaultPosition, wxDefaultSize,
      this->strVidType, wxCB_DROPDOWN | wxCB_READONLY);
   this->strAudioType.Add(_T("MP3 Audio"));
   this->strAudioType.Add(_T("Uncompressed Wave PCM"));
   this->strAudioType.Add(_T("MIB SStar Audio"));
   this->cmbAudio = new wxComboBox(this, ID_CmbAudio, _T(""), wxDefaultPosition, wxDefaultSize,
      this->strAudioType, wxCB_DROPDOWN | wxCB_READONLY);
   this->strDescType.Add(_T("UltraStar TXT"));
   this->strDescType.Add(_T("SStar XML"));
   this->cmbDesc = new wxComboBox(this, ID_CmbDesc, _T(""), wxDefaultPosition, wxDefaultSize,
      this->strDescType, wxCB_DROPDOWN | wxCB_READONLY);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Video File:")), 0);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Audio File:")), 0);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Description File:")), 0);
   lyT->Add(cmbVid, 0);
   lyT->Add(cmbAudio, 0);
   lyT->Add(cmbDesc, 0);

   ///header files (only SStar files)
   lyB = new wxStaticBoxSizer(wxVERTICAL, this, _T("Header Files (Only for SStar Audio/Video)"));
   lyM->Add(lyB, 0, wxEXPAND | wxALL, 5);
   lyT = new wxFlexGridSizer(2, 3, 5, 5);
   lyT->AddGrowableCol(1, 1);
   lyB->Add(lyT, 0, wxEXPAND | wxALL, 5);
   this->txtVidHeader = new wxTextCtrl(this, wxID_ANY);
   this->txtVidHeader->Disable();
   this->txtAudioHeader = new wxTextCtrl(this, wxID_ANY);
   this->txtAudioHeader->Disable();
   this->btnVidHeader = new wxButton(this, ID_BtnVidHeader, _T("..."));
   this->btnVidHeader->Enable(this->Settings->nVidType == ::SettingsC::VIDEOTYPE_IPU);
   this->btnAudioHeader = new wxButton(this, ID_BtnAudioHeader, _T("..."));
   this->btnAudioHeader->Enable(this->Settings->nAudioType == ::SettingsC::AUDIOTYPE_MIB);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("IDX Video:")), 0);
   lyT->Add(txtVidHeader, 1, wxGROW);
   lyT->Add(btnVidHeader, 0);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("MIH Audio:")), 0);
   lyT->Add(txtAudioHeader, 1, wxGROW);
   lyT->Add(btnAudioHeader, 0);

   ///title, artist
   lyB = new wxStaticBoxSizer(wxVERTICAL, this, _T("Track Information:"));
   lyM->Add(lyB, 0, wxEXPAND | wxALL, 5);
   lyT = new wxFlexGridSizer(2, 2, 5, 5);
   lyT->AddGrowableCol(1, 1);
   lyB->Add(lyT, 0, wxEXPAND | wxALL, 5);
   this->cmbArtist = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize,
      0, NULL, wxCB_DROPDOWN);
   this->cmbTitle = new wxComboBox(this, wxID_ANY, _T(""), wxDefaultPosition, wxDefaultSize,
      0, NULL, wxCB_DROPDOWN);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Title:")), 0);
   lyT->Add(this->cmbTitle, 1, wxGROW);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Artist:")), 0);
   lyT->Add(this->cmbArtist, 1, wxGROW);

   ///advanced options
   lyB = new wxStaticBoxSizer(wxVERTICAL, this, _T("Advanced Options"));
   lyM->Add(lyB, 0, wxEXPAND | wxALL, 5);
   lyT = new wxFlexGridSizer(2, 2, 5, 5);
   lyT->AddGrowableCol(0, 1);
   lyB->Add(lyT, 0, wxEXPAND | wxALL, 5);
   this->txtInterleave = new wxTextCtrl(this, wxID_ANY, _T("B800"));
   this->txtFrequ = new wxTextCtrl(this, wxID_ANY, _T("B800"));
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Stereo Interleave Value (Hex):")), 1, wxGROW);
   lyT->Add(this->txtInterleave, 0);
   lyT->Add(new wxStaticText(this, wxID_ANY, _T("Frequency (Hex):")), 1, wxGROW);
   lyT->Add(this->txtFrequ, 0);

   ///dialog buttons
   lyM->Add(this->CreateButtonSizer(wxOK | wxCANCEL), 0, wxALIGN_BOTTOM | wxALIGN_RIGHT | wxALL, 5);

   lyM->SetSizeHints(this);
   lyM->Fit(this);
   SetSizer(lyM);
}

///EVENT HANDLERS

void dlgSettingsC::OnChkUseAudioFromVid(wxCommandEvent& event) {
   bool bUseAudioFromVid = this->chkUseAudioFromVid->GetValue();
   this->txtAudio->Enable(!bUseAudioFromVid);
   this->btnAudio->Enable(!bUseAudioFromVid);
   this->cmbAudio->Enable(!bUseAudioFromVid);
   this->OnCmbAudio(event);
   if(bUseAudioFromVid) {
      this->txtAudio->Clear();
      this->txtAudioHeader->Clear();
   }
}

void dlgSettingsC::OnBtnVid(wxCommandEvent& event) {
   wxFileDialog* dlgFil = new wxFileDialog(
      this, _T("Select Video File"), _T(""), _T("*.mpeg;*.mpg"), _T("MPEG Video (*.mpeg; *.mpg)|")
      _T("*.mpeg;*.mpg|AVI Video (*.avi)|*.avi|SStar IPU Video (*.ipu)|*.ipu|All Files|*.*")
   );
   if(dlgFil->ShowModal())
      this->txtVid->SetValue(dlgFil->GetPath());
   //todo: auto detect file type
}

void dlgSettingsC::OnBtnAudio(wxCommandEvent& event) {
   if(this->chkUseAudioFromVid->GetValue()) return;
   wxFileDialog* dlgFil = new wxFileDialog(
      this, _T("Select Audio File"), _T(""), _T("*.mp3"), _T("MPEG Layer 3 (*.mp3)|")
      _T("*.mp3|WAVE Audio (*.wav)|*.wav|SStar MIB Audio (*.mib)|*.mib|All Files|*.*")
   );
   if(dlgFil->ShowModal())
      this->txtAudio->SetValue(dlgFil->GetPath());
   //todo: auto detect file type
}

void dlgSettingsC::OnBtnDesc(wxCommandEvent& event) {
   wxFileDialog* dlgFil = new wxFileDialog(
      this, _T("Select Description File"), _T(""), _T("*.txt"), _T("UltraStar TXT (*.txt)|")
      _T("*.txt|Singstar XML (*.xml)|*.xml|All Files|*.*")
   );
   if(dlgFil->ShowModal())
      this->txtDesc->SetValue(dlgFil->GetPath());
   //todo: auto detect file type
}

void dlgSettingsC::OnCmbVid(wxCommandEvent& event) {
   if(this->cmbVid->GetValue() != this->strVidType[::SettingsC::VIDEOTYPE_IPU]) {
      this->txtVidHeader->Clear();
      this->txtVidHeader->Disable();
      this->btnVidHeader->Disable();
      this->chkUseAudioFromVid->Enable(true);
   } else {
      this->txtVidHeader->Enable(true);
      this->btnVidHeader->Enable(true);
      this->chkUseAudioFromVid->SetValue(false);
      wxCommandEvent event;
      this->OnChkUseAudioFromVid(event);
      this->chkUseAudioFromVid->Disable();
   }
}

void dlgSettingsC::OnCmbAudio(wxCommandEvent& event) {
   int test;
   test = test;
   if(this->cmbAudio->GetValue() == this->strAudioType[::SettingsC::AUDIOTYPE_MIB]
      && !(this->chkUseAudioFromVid->GetValue()))
   {
      this->txtAudioHeader->Enable(true);
      this->btnAudioHeader->Enable(true);
   } else {
      this->txtAudioHeader->Clear();
      this->txtAudioHeader->Disable();
      this->btnAudioHeader->Disable();
   }
   if(this->cmbAudio->GetValue() == this->strAudioType[::SettingsC::AUDIOTYPE_MIB]) {
      this->txtInterleave->Disable();
      this->txtFrequ->Disable();
   } else {
      this->txtInterleave->Enable(true);
      this->txtFrequ->Enable(true);
   }
   //todo... much todo -.-
   ///information from id3 tags
   if(this->cmbAudio->GetValue() == this->strAudioType[::SettingsC::AUDIOTYPE_MP3] &&
      wxFileName().FileExists(this->txtAudio->GetValue()))
   {
      audio::Mp3DecC audioDec(this->txtAudio->GetValue());
      audioDec.ReadHeader();

      this->ctlAppendItemIfNotInList(this->cmbTitle, audioDec.strId3Title);   //title
      this->ctlAppendItemIfNotInList(this->cmbArtist, audioDec.strId3Artist); //artist

      //todo: sample rate, channel count, etc...
   }
}

void dlgSettingsC::OnCmbDesc(wxCommandEvent& event) {
   ///information from ultrastar description file
   if(wxFileName().FileExists(this->txtDesc->GetValue())) {
      if(this->cmbDesc->GetValue() == this->strDescType[::SettingsC::DESCTYPE_TEXT]) {
         desc::TextParserC descTxtParser(this->txtDesc->GetValue());
         descTxtParser.ReadHeader();

         this->ctlAppendItemIfNotInList(this->cmbTitle, descTxtParser.strTitle);   //title
         this->ctlAppendItemIfNotInList(this->cmbArtist, descTxtParser.strArtist); //artist
      } else if(this->cmbDesc->GetValue() == this->strDescType[::SettingsC::DESCTYPE_XML]) {
         desc::XmlParserC descXmlParser(this->txtDesc->GetValue());
         descXmlParser.ReadHeader();

         this->ctlAppendItemIfNotInList(this->cmbArtist, descXmlParser.strArtist);
         wxLogMessage(_T("%s"), _T("gui::dlgSettingsC::OnCmbDesc():"));
         wxLogMessage(_T("\tartist suggestion from xml file: %s"),
                      descXmlParser.strArtist.c_str());
      }
   }

   //todo: read information from xml file
}

void dlgSettingsC::OnBtnVidHeader(wxCommandEvent& event) {
   wxFileDialog* dlgFile = new wxFileDialog(
      this, _T("Select IDX Header"), _T(""), _T("*.idx"), _T("IPU Video Header (*.idx)|*.idx|")
      _T("All Files|*.*")
   );
   if(dlgFile->ShowModal())
      this->txtVidHeader->SetValue(dlgFile->GetPath());
}

void dlgSettingsC::OnBtnAudioHeader(wxCommandEvent& event) {
   wxFileDialog* dlgFile = new wxFileDialog(
      this, _T("Select MIH Header"), _T(""), _T("*.idx"), _T("MIB Audio Header (*.mih)|*.mih|")
      _T("All Files|*.*")
   );
   if(dlgFile->ShowModal())
      this->txtAudioHeader->SetValue(dlgFile->GetPath());
}

void dlgSettingsC::OnBtnOK(wxCommandEvent& event) {
   SaveSettings();
   this->EndModal(wxID_OK);
}

void dlgSettingsC::OnBtnApply(wxCommandEvent& event) {
   SaveSettings();
}

void dlgSettingsC::OnBtnCancel(wxCommandEvent& event) {
   this->EndModal(wxID_CANCEL);
}


///MEMBERS

void dlgSettingsC::ctlAppendItemIfNotInList(wxControlWithItems* ctrl, const wxString& item) {
   if(!item.IsEmpty()) {
      for(unsigned int i = 0; i < ctrl->GetCount(); i++) {
         if(ctrl->GetString(i) == item)
            return;
      }
      ctrl->Append(item);
   }
}

void dlgSettingsC::SetSettings(::SettingsC* settings) {
   this->txtVid->             SetValue(settings->fnmVid.GetFullPath());
   this->txtAudio->           SetValue(settings->fnmAudio.GetFullPath());
   this->txtDesc->            SetValue(settings->fnmDesc.GetFullPath());
   this->txtVidHeader->       SetValue(settings->fnmVidHeader.GetFullPath());
   this->txtAudioHeader->     SetValue(settings->fnmAudioHeader.GetFullPath());
   this->cmbTitle->           SetValue(settings->strTitle);
   this->cmbArtist->          SetValue(settings->strArtist);
   this->chkUseAudioFromVid-> SetValue(settings->bUseAudioFromVid);
   if(settings->nVidType >= 0 && settings->nVidType < SettingsC::VIDEOTYPE_ENUM_LEN)
      this->cmbVid->          SetValue(this->strVidType[settings->nVidType]);
   if(settings->nAudioType >= 0 && settings->nAudioType < SettingsC::AUDIOTYPE_ENUM_LEN)
      this->cmbAudio->        SetValue(this->strAudioType[settings->nAudioType]);
   if(settings->nDescType >= 0 && settings->nDescType < SettingsC::DESCTYPE_ENUM_LEN)
      this->cmbDesc->         SetValue(this->strDescType[settings->nDescType]);

   wxCommandEvent event;
   this->OnChkUseAudioFromVid(event);
   this->OnCmbVid(event);
   this->OnCmbAudio(event);

   this->Settings = settings;
}

::SettingsC* dlgSettingsC::GetSettings() {
   return this->Settings;
}

void dlgSettingsC::SaveSettings() {
   this->Settings->fnmVid           = wxFileName(this->txtVid->GetValue());
   this->Settings->fnmAudio         = wxFileName(this->txtAudio->GetValue());
   this->Settings->fnmDesc          = wxFileName(this->txtDesc->GetValue());
   this->Settings->nVidType         = (::SettingsC::VIDEOTYPE) this->cmbVid->GetCurrentSelection();
   this->Settings->nAudioType       = (::SettingsC::AUDIOTYPE) this->cmbAudio->GetCurrentSelection();
   this->Settings->nDescType        = (::SettingsC::DESCTYPE) this->cmbDesc->GetCurrentSelection();
   this->Settings->fnmVidHeader     = wxFileName(this->txtVidHeader->GetValue());
   this->Settings->fnmAudioHeader   = wxFileName(this->txtAudioHeader->GetValue());
   this->Settings->strTitle         = this->cmbTitle->GetValue();
   this->Settings->strArtist        = this->cmbArtist->GetValue();
   this->Settings->bUseAudioFromVid = this->chkUseAudioFromVid->GetValue();
}

}
