#if !defined(SSTARMOD_GUI_H)
#define SSTARMOD_GUI_H

namespace gui {

///MAIN WINDOW

class frmMain : public wxFrame {
   public:
      ///constructor, destructor
      frmMain(const wxString& title);

      ///controls
      wxTextCtrl*    txtPAK;
      wxButton*      btnPAK;
      wxRadioButton* radPAK;
      wxTextCtrl*    txtDest;
      wxButton*      btnDest;
      wxListCtrl*    lstSrc;

      ///event handlers
      void OnBtnPAK     (wxCommandEvent& event);
      void OnRadPAK     (wxCommandEvent& event);
      void OnBtnDest    (wxCommandEvent& event);
      void OnBtnSrcAdd  (wxCommandEvent& event);
      void OnBtnSrcDel  (wxCommandEvent& event);
      void OnBtnSrcUp   (wxCommandEvent& event);
      void OnBtnSrcDown (wxCommandEvent& event);
      void OnLstSrc     (wxListEvent&    event);
      void OnLstSrcSel  (wxListEvent&    event);

      ///IDs
      enum {
         ID_BtnPAK,
         ID_RadPAK,
         ID_BtnDest,
         ID_BtnSrcAdd,
         ID_BtnSrcDel,
         ID_BtnSrcUp,
         ID_BtnSrcDown,
         ID_LstSrc,
         ID_LstSrcSel,
      };

      ///members

      bool bRadPAKEx;  //folder contains extraction of PAK files
      long nLstSrcSel; //selected item

      void lstSrcRefresh(); //refresh source list

   private:
      DECLARE_EVENT_TABLE()
};
//c++ does not allow this to be member of class frmMain:
int wxCALLBACK lstSrcCmp(long item1, long item2, long sortData); //sorting callback function

///SETTINGS DIALOG
class dlgSettingsC : public wxDialog {
   public:
      ///constructor, destructor
      dlgSettingsC(wxWindow* parent, wxWindowID id, const wxString& title,
                   const wxPoint& pos = wxDefaultPosition, const wxSize& size = wxDefaultSize,
                   long style = wxDEFAULT_DIALOG_STYLE, const wxString& name = _T("Settings"));
      //~dlgSettingsC() { delete this->Settings; }

      ///members
      ::SettingsC* GetSettings();
      void         SetSettings(::SettingsC* settings);

   private:
      ///controls
      wxComboBox* cmbTitle;
      wxComboBox* cmbArtist;
      wxCheckBox* chkUseAudioFromVid;
      wxTextCtrl* txtVid;              //video path
      wxTextCtrl* txtAudio;            //audio path
      wxTextCtrl* txtDesc;             //description file path
      wxButton*   btnAudio;            //select audio path button
      wxComboBox* cmbVid;              //video type
      wxComboBox* cmbAudio;            //audio type
      wxComboBox* cmbDesc;             //description file type
      wxTextCtrl* txtVidHeader;        //IPU video file header
      wxTextCtrl* txtAudioHeader;      //MIB audio file header
      wxButton*   btnVidHeader;        //select video header path button
      wxButton*   btnAudioHeader;      //select audio header path button
      wxTextCtrl* txtInterleave;       //interleave value
      wxTextCtrl* txtFrequ;            //frequency in Hz

      ///event handlers
      void OnChkUseAudioFromVid (wxCommandEvent& event);
      void OnBtnVid             (wxCommandEvent& event);
      void OnBtnAudio           (wxCommandEvent& event);
      void OnBtnDesc            (wxCommandEvent& event);
      void OnCmbVid             (wxCommandEvent& event);
      void OnCmbAudio           (wxCommandEvent& event);
      void OnCmbDesc            (wxCommandEvent& event);
      void OnBtnVidHeader       (wxCommandEvent& event);
      void OnBtnAudioHeader     (wxCommandEvent& event);
      void OnBtnOK              (wxCommandEvent& event);
      void OnBtnApply           (wxCommandEvent& event);
      void OnBtnCancel          (wxCommandEvent& event);

      ///IDs
      enum {
         ID_ChkUseAudioFromVid,
         ID_BtnAudio,
         ID_BtnVid,
         ID_BtnDesc,
         ID_CmbVid,
         ID_CmbAudio,
         ID_CmbDesc,
         ID_BtnVidHeader,
         ID_BtnAudioHeader,

         ID_BtnOK     = wxID_OK,
         ID_BtnApply  = wxID_APPLY,
         ID_BtnCancel = wxID_CANCEL,
      };

      ///members

      void ctlAppendItemIfNotInList(wxControlWithItems* ctrl, const wxString& item);
      void SaveSettings();
      ::SettingsC* Settings;

      //entries in filetype comboboxes
      wxArrayString strVidType;
      wxArrayString strAudioType;
      wxArrayString strDescType;


      DECLARE_EVENT_TABLE()
};

extern gui::frmMain* frame;

}

#endif //SSTARMOD_GUI_H
